@@ -814,16 +814,6 @@ ULONG WINAPI TraceEvent( TRACEHANDLE SessionHandle, PEVENT_TRACE_HEADER EventTra
     return ERROR_CALL_NOT_IMPLEMENTED;
 }
 
-/******************************************************************************
- * EventEnabled [ADVAPI32.@]
- *
- */
-BOOLEAN WINAPI EventEnabled( REGHANDLE handle, PCEVENT_DESCRIPTOR descriptor )
-{
-    FIXME("(%s, %p): stub\n", wine_dbgstr_longlong(handle), descriptor);
-    return FALSE;
-}
-
 /******************************************************************************
  * EventProviderEnabled [ADVAPI32.@]
  *
@@ -844,16 +834,6 @@ ULONG WINAPI EventActivityIdControl(ULONG code, GUID *guid)
     return ERROR_SUCCESS;
 }
 
-/******************************************************************************
- * EventWrite [ADVAPI32.@]
- */
-ULONG WINAPI EventWrite( REGHANDLE handle, PCEVENT_DESCRIPTOR descriptor, ULONG count,
-                         PEVENT_DATA_DESCRIPTOR data )
-{
-    FIXME("%s, %p, %u, %p: stub\n", wine_dbgstr_longlong(handle), descriptor, count, data);
-    return ERROR_SUCCESS;
-}
-
 /******************************************************************************
  * EventWriteTransfer [ADVAPI32.@]
  */
