@@ -257,12 +257,10 @@ static inline BOOL MONTHCAL_IsDateEqual(const SYSTEMTIME *first, const SYSTEMTIM
 /* make sure that date fields are valid */
 static BOOL MONTHCAL_ValidateDate(const SYSTEMTIME *time)
 {
-    if (time->wMonth < 1 || time->wMonth > 12 )
-        return FALSE;
-    if (time->wDay == 0 || time->wDay > MONTHCAL_MonthLength(time->wMonth, time->wYear))
-        return FALSE;
+  if(time->wMonth < 1 || time->wMonth > 12 ) return FALSE;
+  if(time->wDay > MONTHCAL_MonthLength(time->wMonth, time->wYear)) return FALSE;
 
-    return TRUE;
+  return TRUE;
 }
 
 /* Copies timestamp part only.
@@ -1740,28 +1738,21 @@ MONTHCAL_GetToday(const MONTHCAL_INFO *infoPtr, SYSTEMTIME *today)
 static BOOL
 MONTHCAL_UpdateToday(MONTHCAL_INFO *infoPtr, const SYSTEMTIME *today)
 {
-    RECT rect;
+  RECT new_r, old_r;
 
-    if (MONTHCAL_IsDateEqual(today, &infoPtr->todaysDate))
-        return FALSE;
+  if(MONTHCAL_IsDateEqual(today, &infoPtr->todaysDate)) return FALSE;
 
-    /* Invalidate old and new today day rectangle, and today label. */
-    if (MONTHCAL_ValidateDate(&infoPtr->todaysDate))
-    {
-        MONTHCAL_GetDayRect(infoPtr, &infoPtr->todaysDate, &rect, -1);
-        InvalidateRect(infoPtr->hwndSelf, &rect, FALSE);
-    }
+  MONTHCAL_GetDayRect(infoPtr, &infoPtr->todaysDate, &old_r, -1);
+  MONTHCAL_GetDayRect(infoPtr, today, &new_r, -1);
 
-    if (MONTHCAL_ValidateDate(today))
-    {
-        MONTHCAL_GetDayRect(infoPtr, today, &rect, -1);
-        InvalidateRect(infoPtr->hwndSelf, &rect, FALSE);
-    }
+  infoPtr->todaysDate = *today;
 
-    infoPtr->todaysDate = *today;
-
-    InvalidateRect(infoPtr->hwndSelf, &infoPtr->todayrect, FALSE);
-    return TRUE;
+  /* only two days need redrawing */
+  InvalidateRect(infoPtr->hwndSelf, &old_r, FALSE);
+  InvalidateRect(infoPtr->hwndSelf, &new_r, FALSE);
+  /* and today label */
+  InvalidateRect(infoPtr->hwndSelf, &infoPtr->todayrect, FALSE);
+  return TRUE;
 }
 
 /* MCM_SETTODAT handler */
@@ -1916,7 +1907,6 @@ MONTHCAL_HitTest(const MONTHCAL_INFO *infoPtr, MCHITTESTINFO *lpht)
   else if(PtInRect(&infoPtr->calendars[calIdx].days, lpht->pt))
   {
       htinfo.iOffset = calIdx;
-      htinfo.st.wDay = ht_month->wDay;
       htinfo.st.wYear  = ht_month->wYear;
       htinfo.st.wMonth = ht_month->wMonth;
       /* previous month only valid for first calendar */
