@@ -100,6 +100,8 @@ static inline struct d2d_wic_render_target *impl_from_ID2D1RenderTarget(ID2D1Ren
 
 static HRESULT STDMETHODCALLTYPE d2d_wic_render_target_QueryInterface(ID2D1RenderTarget *iface, REFIID iid, void **out)
 {
+    struct d2d_wic_render_target *render_target = impl_from_ID2D1RenderTarget(iface);
+
     TRACE("iface %p, iid %s, out %p.\n", iface, debugstr_guid(iid), out);
 
     if (IsEqualGUID(iid, &IID_ID2D1RenderTarget)
@@ -110,6 +112,8 @@ static HRESULT STDMETHODCALLTYPE d2d_wic_render_target_QueryInterface(ID2D1Rende
         *out = iface;
         return S_OK;
     }
+    else if (IsEqualGUID(iid, &IID_ID2D1GdiInteropRenderTarget))
+        return ID2D1RenderTarget_QueryInterface(render_target->dxgi_target, iid, out);
 
     WARN("%s not implemented, returning E_NOINTERFACE.\n", debugstr_guid(iid));
 
@@ -841,7 +845,8 @@ HRESULT d2d_wic_render_target_init(struct d2d_wic_render_target *render_target,
     texture_desc.Usage = D3D10_USAGE_DEFAULT;
     texture_desc.BindFlags = D3D10_BIND_RENDER_TARGET | D3D10_BIND_SHADER_RESOURCE;
     texture_desc.CPUAccessFlags = 0;
-    texture_desc.MiscFlags = 0;
+    texture_desc.MiscFlags = desc->usage & D2D1_RENDER_TARGET_USAGE_GDI_COMPATIBLE ?
+            D3D10_RESOURCE_MISC_GDI_COMPATIBLE : 0;
 
     if (FAILED(hr = ID3D10Device1_CreateTexture2D(device, &texture_desc, NULL, &texture)))
     {
@@ -860,6 +865,7 @@ HRESULT d2d_wic_render_target_init(struct d2d_wic_render_target *render_target,
     texture_desc.Usage = D3D10_USAGE_STAGING;
     texture_desc.BindFlags = 0;
     texture_desc.CPUAccessFlags = D3D10_CPU_ACCESS_READ;
+    texture_desc.MiscFlags = 0;
 
     if (FAILED(hr = ID3D10Device1_CreateTexture2D(device, &texture_desc, NULL, &render_target->readback_texture)))
     {
@@ -868,8 +874,8 @@ HRESULT d2d_wic_render_target_init(struct d2d_wic_render_target *render_target,
         return hr;
     }
 
-    if (FAILED(hr = ID2D1Factory_CreateDxgiSurfaceRenderTarget(factory,
-            render_target->dxgi_surface, desc, &render_target->dxgi_target)))
+    if (FAILED(hr = d2d_d3d_create_render_target(factory, render_target->dxgi_surface,
+            (IUnknown *)&render_target->ID2D1RenderTarget_iface, desc, &render_target->dxgi_target)))
     {
         WARN("Failed to create DXGI surface render target, hr %#x.\n", hr);
         ID3D10Texture2D_Release(render_target->readback_texture);
