@@ -44,6 +44,8 @@ enum wined3d_gl_extension
     APPLE_YCBCR_422,
     /* ARB */
     ARB_BLEND_FUNC_EXTENDED,
+    ARB_CLEAR_BUFFER_OBJECT,
+    ARB_CLEAR_TEXTURE,
     ARB_CLIP_CONTROL,
     ARB_COLOR_BUFFER_FLOAT,
     ARB_COMPUTE_SHADER,
